@@ -4,18 +4,18 @@ Basic class handling model tree structure.
 import abc
 import collections
 import logging
-from typing import Any, Optional, List
+from typing import Any, Optional, List, Generator, MappingView, OrderedDict, Sequence
 
+from cosapp.patterns.visitor import Visitor, Component as VisitedComponent
 from cosapp.core.signal import Signal
 from cosapp.utils.naming import NameChecker
 from cosapp.utils.helpers import check_arg
 from cosapp.utils.logging import LoggerContext, LogFormat, LogLevel
-from cosapp.utils.orderedset import OrderedSet
 
 logger = logging.getLogger(__name__)
 
 
-class Module(LoggerContext, metaclass=abc.ABCMeta):
+class Module(LoggerContext, VisitedComponent, metaclass=abc.ABCMeta):
     # class Module does not inherit directly from abc.ABC, due to 
     # a bug in Python 3.6 preventing the use of __weakref__ in slots.
     # Ref: https://bugs.python.org/issue30463
@@ -35,11 +35,11 @@ class Module(LoggerContext, metaclass=abc.ABCMeta):
     name : str
         `Module` name
     children : Dict[str, Module]
-        Child `Module` of this `Module`
+        Sub-modules of current `Module`, referenced by names.
     parent : Module
-        Parent `Module` of this `Module`; None if there is no parent.
-    exec_order : OrderedSet[str]
-        Execution order in which the child `Module` should be solved.
+        Parent `Module` of current `Module`; `None` if there is no parent.
+    exec_order : MappingView[str]
+        Execution order in which sub-modules should be computed.
 
     _active : bool
         If False, the `Module` will not execute its `run_once` method
@@ -57,7 +57,7 @@ class Module(LoggerContext, metaclass=abc.ABCMeta):
     """
 
     __slots__ = (
-        '__weakref__', '_name', 'children', 'parent', '__exec_order', '_active',
+        '__weakref__', '_name', 'children', 'parent', '_active',
         '_compute_calls', 'setup_ran', 'computed', 'clean_ran',
     )
 
@@ -74,9 +74,7 @@ class Module(LoggerContext, metaclass=abc.ABCMeta):
         self._name = self._name_check(name)  # type: str
         self.children = collections.OrderedDict()
         self.parent = None  # type: Optional[Module]
-        self.__exec_order = OrderedSet()  # type: OrderedSet[str]
         self._active = True  # type: bool
-
         self._compute_calls = 0  # type: int
 
         # Signals
@@ -84,6 +82,27 @@ class Module(LoggerContext, metaclass=abc.ABCMeta):
         self.computed = Signal(name="cosapp.core.module.Module.computed")
         self.clean_ran = Signal(name="cosapp.core.module.Module.clean_ran")
 
+    def tree(self, downwards=False) -> Generator["Module", None, None]:
+        """Generator recursively yielding all elements in module tree.
+        
+        Parameters:
+        -----------
+        - downwards [bool, optional]:
+            If `True`, yields elements from top to bottom.
+            If `False` (default), yields elements from bottom to top.
+        """
+        if downwards:
+            yield self
+        for child in self.children.values():
+            yield from child.tree(downwards)
+        if not downwards:
+            yield self
+
+    def send_visitor(self, visitor: Visitor, downwards=False) -> None:
+        """Recursively accept visitor throughout module tree."""
+        for module in self.tree(downwards):
+            module.accept(visitor)
+
     def __getattr__(self, name: str) -> Any:
         try:  # Faster than testing
             return self.children[name]
@@ -110,55 +129,76 @@ class Module(LoggerContext, metaclass=abc.ABCMeta):
         self._name = self._name_check(name)
 
     @property
-    def exec_order(self) -> "OrderedSet[str]":
-        return self.__exec_order
+    def exec_order(self) -> MappingView[str]:
+        """MappingView[str]: sub-module execution order, as a name iterator"""
+        return self.children.keys()
 
     @exec_order.setter
-    def exec_order(self, iterable) -> None:
-        new_set = OrderedSet(iterable)
-        if not all(isinstance(elem, str) for elem in new_set):
-            raise TypeError(f"All elements of {self.name}.exec_order must be strings")
-        self.__exec_order = new_set
+    def exec_order(self, namelist: Sequence[str]) -> None:
+        if not isinstance(namelist, Sequence):
+            raise TypeError("exec_order must be an ordered sequence of strings")
+        nameset = set(self.children)
+        if set(namelist) != nameset:
+            if nameset:
+                msg = f"exec_order must be a permutation of {list(self.children)}"
+            else:
+                msg = f"Can't set exec_order, as {self.name!r} has no children"
+            logger.error(f"{msg}; got {namelist}.")
+            raise ValueError(msg)
+        elif len(namelist) > len(self.children):
+            repeated = list(namelist)
+            for name in set(namelist):
+                repeated.remove(name)
+            raise ValueError(f"Repeated items {sorted(set(repeated))}")
+        # Rearrange children in a new dictionary
+        self.children = OrderedDict(
+            (name, self.children[name])
+            for name in namelist
+        )
 
     @property
     def size(self) -> int:
-        """int : System size (= number of children + 1)."""
-        size = 1
-        for child in self.children.values():
-            size += child.size
-        return size
+        """int: Total number of elements in tree."""
+        return sum(1 for _ in self.tree())
+
+    def path_to_root(self) -> Generator["Module", None, None]:
+        """Generator recursively yielding all elements up to root module.
+        """
+        current = self
+        yield current
+        while current.parent is not None:
+            current = current.parent
+            yield current
 
     def root(self) -> "Module":
-        return self.path()[0]
+        for root in self.path_to_root():
+            continue
+        return root
 
     def path(self) -> List["Module"]:
-        """Returns full path up to root Module as a list.
+        """Returns full path from root Module as a list.
         
         Returns
         -------
         List[Module]
             Full module list from root to self
         """
-        current = self
-        path = [current]
-        while current.parent is not None:
-            current = current.parent
-            path.append(current)
-        path.reverse()
-        return path
+        path = list(self.path_to_root())
+        return list(reversed(path))
 
     def path_namelist(self) -> List[str]:
-        """Returns full name list up to root Module.
+        """Returns full name list from root Module.
         
         Returns
         -------
         List[str]
             The module full name list
         """
-        return [module.name for module in self.path()]
+        names = [elem.name for elem in self.path_to_root()]
+        return list(reversed(names))
 
     def full_name(self, trim_root=False) -> str:
-        """Returns full name up to root Module.
+        """Returns full name from root Module.
         
         Parameters
         ----------
@@ -241,8 +281,12 @@ class Module(LoggerContext, metaclass=abc.ABCMeta):
         """
         # Type validation
         check_arg(child, 'child', Module)
+
+        specific_order = None
         if execution_index is not None:
             check_arg(execution_index, 'execution_index', int)
+            specific_order = list(self.exec_order)
+            specific_order.insert(execution_index, child.name)
 
         if child.name in self.children:
             raise ValueError(
@@ -254,10 +298,8 @@ class Module(LoggerContext, metaclass=abc.ABCMeta):
         children = self.children
         children[child.name] = child
 
-        if execution_index is None:
-            self.__exec_order.append(child.name)
-        else:
-            self.__exec_order.insert(execution_index, child.name)
+        if specific_order:
+            self.exec_order = specific_order
 
         return child
 
@@ -284,35 +326,8 @@ class Module(LoggerContext, metaclass=abc.ABCMeta):
         child = children.pop(name)
         child.parent = None
 
-        self.__exec_order.remove(name)
-
         return child
 
-    def _set_execution_order(self):
-        # TODO doc + unit test
-
-        # if self.execution_algorithm != ExecutionOrdering.MANUAL:
-        #     pass  # TODO implement alternative
-        # Warning the execution order should not be set for standalone sub-multisystems. They
-        # should take care of them self.
-
-        # Check if all the children are listed in execution order list. Warn if not
-        for key, component in self.children.items():
-            if key not in self.exec_order:
-                logger.info(
-                    f"Missing module '{key}' has been appended to the execution order list."
-                )
-                self.exec_order.add(key)
-            component._set_execution_order()
-
-        if len(self.children) != len(self.exec_order):
-            for c in self.exec_order:
-                if c not in self.children:
-                    logger.info(
-                        f"Module {c} in execution order list without component matching. The entry will be removed."
-                    )
-                    self.exec_order.discard(c)
-
     def _precompute(self) -> None:
         """Actions performed prior to the `Module.compute` call."""
         pass
