@@ -34,21 +34,22 @@ class Scope(IntEnum):
 class Validity(IntEnum):
     """Enumeration for the validity of a variable value.
     
-    A valid variable value is `OK`. If the value is outside the limits, its status is `ERROR`. And
-    if the value falls between the validity range and the limits, its `WARNING`.
+    A valid variable value is `OK`.
+    If the value is outside the limits, status is `ERROR`.
+    If the value falls between the validity range and the limits, status is `WARNING`.
     """
-
     ERROR = -1
     WARNING = 0
     OK = 1
 
+
 class RangeType(IntEnum):
     """Enumeration for the type of valid_range and limit of variable.
     
-    If the value is a 2-tuple, type is `VALUE`. If the value is tuple of tuples, its type is `TUPLE`. And
-    if the value is NONE, its type is `NONE`.
+    If the value is a (lower, upper) tuple, type is `VALUE`.
+    If the value is a tuple of tuples, type is `TUPLE`.
+    Tf the value is None, type is `NONE`.
     """
-
     NONE = 0
     VALUE = 1
     TUPLE = 2
