@@ -9,17 +9,18 @@ from typing import Any, Dict, Iterator, Optional, Tuple, Union
 from types import MappingProxyType
 import numpy
 
-from cosapp.core.numerics.distributions.distribution import Distribution
+from cosapp.patterns import visitor
 from cosapp.ports.enum import PortType, Scope, Validity
 from cosapp.ports.exceptions import ScopeError
 from cosapp.ports.variable import RangeValue, Types, Variable
+from cosapp.utils.distributions import Distribution
 from cosapp.utils.helpers import check_arg
 from cosapp.utils.naming import NameChecker
 
 logger = logging.getLogger(__name__)
 
 
-class BasePort:
+class BasePort(visitor.Component):
     """Base class for ports, containers gathering variables.
 
     Common users should not use this class directly.
@@ -54,6 +55,10 @@ class BasePort:
         self.__clearance = None
         self.scope_clearance = Scope.PRIVATE
 
+    def accept(self, visitor: visitor.Visitor) -> None:
+        """Specifies course of action when visited by `visitor`"""
+        visitor.visit_port(self)
+
     @property
     def owner(self) -> "Optional[cosapp.systems.System]":
         """System : `System` owning the port."""
@@ -338,6 +343,10 @@ class BasePort:
     def __len__(self) -> int:
         return len(self._variables)
 
+    def items(self):
+        for name in self._variables:
+            yield (name, getattr(self, name))
+
     @property
     def scope_clearance(self) -> Scope:
         """Scope: Current clearance level of the port.
